import * as React from 'react';
import { ScopeContext, ForContext, getComponentRenderList, DataContext } from './NodeRenderer';
import { getScopeSlotMeta, isSlot } from '../utils/common';
import { observer } from 'mobx-react-lite';
import { ComponentNotFound } from '../utils/error';
import { InjectContext } from '../router';
import { checkVisible } from '../utils';
import { get, isEqual } from 'lodash-es';
import { autorun } from 'mobx';
function checkDoNotRenderSlot(componentSchema, renderSlot, rootNode) {
    // 判断是否为 slot
    return isSlot(componentSchema) && !(renderSlot || rootNode);
}
export function getComponentChildren(component, context) {
    const { items = [] } = component;
    if (!items.length) {
        return [];
    }
    const list = items.sort((a, b) => { var _a, _b; return (((_a = a.extra) === null || _a === void 0 ? void 0 : _a.xIndex) || 0) - (((_b = b.extra) === null || _b === void 0 ? void 0 : _b.xIndex) || 0); });
    const { virtualFields, codeContext, scopeContext, forContext, injectContext, dataContext, updateContext } = context;
    return list.map((schema) => {
        return getRenderList({
            key: schema.id,
            componentSchema: schema,
            rootNode: false,
            renderSlot: false,
            virtualFields,
            codeContext,
            scopeContext,
            forContext,
            injectContext,
            dataContext,
            updateContext,
        });
    });
}
export const Trace = observer(function Trace({ id }) {
    const scopeContext = React.useContext(ScopeContext);
    console.log('id', id, get(scopeContext, id));
    return null;
});
function useEqualValue(value) {
    const valueRef = React.useRef(value);
    return React.useMemo(() => {
        if (!isEqual(value, valueRef.current)) {
            valueRef.current = value;
        }
        return valueRef.current;
    }, [value]);
}
function useObserverWdigetData(id, codeContext) {
    const [_, forceUpate] = React.useState(Date.now());
    const widgetDataRef = React.useRef(null);
    const isInComposite = !!codeContext.$WEAPPS_COMP;
    widgetDataRef.current = !isInComposite ? codeContext.$page.widgets[id] : codeContext.$WEAPPS_COMP.widgets[id];
    const widgetsData = widgetDataRef.current;
    const dispose = React.useMemo(() => {
        return autorun((r) => {
            // r.trace();
            let widgetData = !isInComposite ? codeContext.$page.widgets[id] : codeContext.$WEAPPS_COMP.widgets[id];
            if (Array.isArray(widgetData)) {
                const delta = widgetData.length || 1;
                forceUpate((_) => {
                    return _ + delta;
                });
            }
        });
    }, []);
    React.useEffect(() => {
        return () => {
            dispose();
        };
    }, []);
    return widgetsData;
}
function HOCWtichRegistData(props) {
    var _a, _b, _c;
    const { id, data: _data, componentSchema, renderSlot, slotConfig = {}, codeContext, forContext: _forContext, scopeContext, injectContext, dataContext, virtualFields, } = props;
    const data = useEqualValue(_data);
    const isMultiSlotInstance = ((_a = slotConfig === null || slotConfig === void 0 ? void 0 : slotConfig['x-slot-directives']) === null || _a === void 0 ? void 0 : _a._index) && ((_b = slotConfig === null || slotConfig === void 0 ? void 0 : slotConfig['x-slot-directives']) === null || _b === void 0 ? void 0 : _b.for);
    const indexKey = (_c = slotConfig === null || slotConfig === void 0 ? void 0 : slotConfig['x-slot-directives']) === null || _c === void 0 ? void 0 : _c._index;
    const widgetsData = useObserverWdigetData(id, codeContext);
    let forContext = React.useMemo(() => {
        const { forIndexes = [], forStack = [] } = _forContext;
        return isMultiSlotInstance && data[indexKey] !== undefined
            ? Object.assign(Object.assign({}, _forContext), { [id]: data, forIndexes: forIndexes.concat(data[indexKey]), forStack: [
                    ...forStack,
                    {
                        currentIndex: data[indexKey],
                        currentItem: data,
                    },
                ] }) : _forContext;
    }, [data[indexKey]]);
    const { forIndexes = [] } = forContext;
    const currentWidget = Array.isArray(widgetsData)
        ? widgetsData.length
            ? get(widgetsData, forIndexes)
            : undefined
        : widgetsData;
    const [componentId, key] = id.split('.');
    React.useEffect(() => {
        if (!currentWidget) {
            // debugger;
            return;
        }
        currentWidget._scope.scopeContext = data;
    }, [currentWidget, data]);
    const memoScopeContext = React.useMemo(() => {
        return Object.assign(Object.assign({}, scopeContext), { [componentId]: {
                [key]: data,
            } });
    }, [componentId, key, data]);
    return currentWidget ? (React.createElement(InjectContext.Provider, { value: injectContext },
        React.createElement(ScopeContext.Provider, { value: memoScopeContext },
            React.createElement(DataContext.Provider, { value: dataContext },
                React.createElement(ForContext.Provider, { value: forContext },
                    React.createElement(AppRender, { componentSchema: componentSchema, renderSlot: renderSlot, virtualFields: virtualFields, codeContext: codeContext })))))) : null;
}
// const ObserverableHOCWtichRegistData = observer(HOCWtichRegistData);
const ObserverableHOCWtichRegistData = HOCWtichRegistData;
export function generateSlotMetaMap(componentSchema, context, options) {
    const { items = [], id, module, component } = componentSchema;
    const { scopeContext, codeContext, virtualFields, injectContext } = context;
    const slots = {};
    // eslint-disable-next-line guard-for-in
    for (const child of items) {
        // TODO 做到不依赖组件 id
        const key = child.id;
        if (isSlot(child)) {
            const slotId = `${id}.${key}`;
            if ((injectContext.checkVisible || checkVisible)({ id: slotId })) {
                const { isScopeSlot: isHOC, config } = getScopeSlotMeta({ module, component }, key, injectContext);
                slots[key] = {
                    type: isHOC ? 'HOC' : 'ELEMENT',
                    node: isHOC
                        ? function ({ forContext, dataContext }) {
                            return function ScopedSlot(props) {
                                return (React.createElement(ObserverableHOCWtichRegistData, { id: slotId, data: props, componentSchema: child, renderSlot: options.renderSlot, slotConfig: config, codeContext: codeContext, forContext: forContext, dataContext: dataContext, scopeContext: scopeContext, injectContext: injectContext, virtualFields: virtualFields }));
                            };
                        }
                        : ({ forContext, dataContext }) => {
                            return (React.createElement(InjectContext.Provider, { value: injectContext },
                                React.createElement(ScopeContext.Provider, { value: scopeContext },
                                    React.createElement(DataContext.Provider, { value: dataContext },
                                        React.createElement(ForContext.Provider, { value: forContext },
                                            React.createElement(AppRender, { componentSchema: child, renderSlot: options === null || options === void 0 ? void 0 : options.renderSlot, virtualFields: virtualFields, codeContext: codeContext }))))));
                        },
                };
            }
        }
    }
    return slots;
}
function getRenderList(props) {
    var _a;
    const { key, virtualFields, componentSchema, renderSlot, rootNode = true, codeContext, scopeContext, forContext = {}, injectContext = {}, dataContext, updateContext, } = props;
    const { genericComp = {} } = componentSchema.extra || {};
    // 判断是否为 slot
    if (checkDoNotRenderSlot(componentSchema, renderSlot, rootNode)) {
        return null;
    }
    if (componentSchema.module && componentSchema.component) {
        const sourceKey = `${componentSchema.module}:${componentSchema.component}`;
        const Field = virtualFields[sourceKey];
        if (!Field) {
            return React.createElement(ComponentNotFound, { key: key, module: componentSchema.module, component: componentSchema.component });
        }
    }
    return getComponentRenderList({
        key,
        componentSchema,
        rootNode,
        emitEvents: ((_a = componentSchema.extra) === null || _a === void 0 ? void 0 : _a.emitEvents) || [],
        renderSlot,
        virtualFields,
        codeContext,
        scopeContext,
        forContext,
        dataContext,
        updateContext,
        injectContext,
        genericComp,
    });
}
// CommonAppRender.whyDidYouRender = true;
export function CommonAppRender(props) {
    var _a, _b, _c, _d;
    // trace(true);
    const { className, componentSchema } = props;
    //  wrapperClass;
    const preClassNameList = React.useRef([]);
    const { items = [] } = componentSchema;
    const currentInjectClassNameList = ((_c = (_b = (_a = className === null || className === void 0 ? void 0 : className.split) === null || _a === void 0 ? void 0 : _a.call(className, /\s/)) === null || _b === void 0 ? void 0 : _b.filter) === null || _c === void 0 ? void 0 : _c.call(_b, (item) => !!item)) || [];
    const containerEl = items[0];
    if (className && containerEl) {
        if (!containerEl.attributes) {
            containerEl.attributes = {};
        }
        let classNameList = (((_d = containerEl.attributes) === null || _d === void 0 ? void 0 : _d.class) || '').split(/\s/).filter((item) => !!item);
        // 先替换掉先前计算出来的className部分
        if (preClassNameList.current.length) {
            const map = preClassNameList.current.reduce((map, item) => {
                map[item] = true;
                return map;
            }, {});
            classNameList = classNameList.filter((item) => !map[item]);
        }
        containerEl.attributes.class = Array.from(new Set(classNameList.concat(currentInjectClassNameList))).join(' ');
    }
    preClassNameList.current = currentInjectClassNameList;
    return getRenderList(Object.assign(Object.assign({}, props), { 
        // dataContext: {},
        // scopeContext: {},
        // forContext: {},
        // injectContext: {},
        dataContext: React.useContext(DataContext), scopeContext: React.useContext(ScopeContext), forContext: React.useContext(ForContext), injectContext: React.useContext(InjectContext) }));
}
export const AppRender = observer(CommonAppRender);
