import { __awaiter } from "tslib";
import { kebabCase } from './common';
import { PostcssRpx2Calc } from '@cloudbase/cals';
import { CALC_RPX_ROOT_VAL_NAME, postcssImport } from './index';
export function toCssText(style, className = '.some-class-name') {
    const attrText = Object.keys(style)
        .map((key) => {
        const value = style[key];
        return `${kebabCase(key)}: ${value};`;
    })
        .join('\n');
    return `${className} { ${attrText} }\n`;
}
export function generateScopedStyleText(schema, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const cssTextList = [];
        function traverse(item) {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                if (item.scopedStyle && item.id) {
                    let cssText = item.scopedStyle
                        // 移除 `/* ... */` 注释
                        .replace(/\/\*(?:[\s\S]*?)\*\//g, '')
                        // 移除换行及其前后的空白符
                        .replace(/\s*\n\s*/g, '');
                    if (options === null || options === void 0 ? void 0 : options.enablePageRootId) {
                        // 添加 `#wd-page-root` 前缀以增加样式权重
                        cssText = cssText.replace(/(:scope.*?[,{])/g, '\n#wd-page-root $1');
                    }
                    cssText = cssText
                        // 替换 `:scope`
                        .replace(/:scope/g, `.wd-comp-id-${item.id}`)
                        // `: ` => `:`
                        // `, ` => `,`
                        // ` {` => `{`
                        .replace(/\s*([:,{])\s*/g, '$1')
                        // `;}` => `}`
                        .replace(/;\s*}/g, '}');
                    if (cssText.indexOf('rpx') !== -1) {
                        const postcss = yield postcssImport();
                        const { css } = yield postcss([PostcssRpx2Calc({ root: `var(${CALC_RPX_ROOT_VAL_NAME})` })]).process(cssText);
                        cssText = css;
                    }
                    cssTextList.push(cssText.trim());
                }
                yield Promise.all(((_b = (_a = item.items) === null || _a === void 0 ? void 0 : _a.map) === null || _b === void 0 ? void 0 : _b.call(_a, (childItem) => traverse(childItem))) || []);
            });
        }
        yield traverse(schema);
        return cssTextList.join('\n');
    });
}
export function mergeCssVarConfig(cssVarConfig = {}, platform = 'PCWEB') {
    let merged = Object.assign({}, cssVarConfig === null || cssVarConfig === void 0 ? void 0 : cssVarConfig._);
    let list = [platform];
    switch (platform) {
        case 'PCWEB': {
            list = ['WEB', 'PCWEB'];
            break;
        }
        case 'MOBILEWEB': {
            list = ['WEB', 'MOBILE', 'MOBILEWEB'];
            break;
        }
        case 'MP': {
            list = ['MOBILE', 'MP'];
            break;
        }
        case 'WEB':
        case 'MOBILE': {
            list = [platform];
        }
    }
    for (let key of list) {
        if (cssVarConfig[key]) {
            merged = Object.assign(Object.assign({}, merged), cssVarConfig[key]);
        }
    }
    return merged;
}
